/*
** iplog_ident.c - iplog IDENT lookup routines.
** Copyright (C) 1999 behe <eric@ojnk.net>, Odin <odin@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
*/

#include <config.h>

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <sys/time.h>
#include <sys/types.h>
#define __FAVOR_BSD
#include <sys/socket.h>

#include <iplog.h>

#ifndef __linux__
bool is_listening(port_t unused) {
	(void) unused;
	return (false);
}

void *get_ident_data(void *unused) {
	(void) unused;
	return (NULL);
}
#else

#define TCP_FORMAT "%*d: %*X:%x %*X:%*x %x %*X:%*X %*x:%*X %*x %*d %*d %*d"
#define TCP_LISTEN	0x0A
#define TCP_DATA	"/proc/net/tcp"

void *get_ident_data(void *data) {
	int sock;
	struct sockaddr_in sin;
	u_char buf[128], temp[64], lbuf[MAX_HSTLEN], sbuf[MAX_SRVLEN];
	ssize_t len, blen;
	struct ip *ip = (struct ip *) data; 
	struct tcphdr *tcp = (struct tcphdr *) ((struct ip *) ip + 1);

	sock = socket(AF_INET, SOCK_STREAM, 0);

	if (sock == -1)
		goto ident_fail;

	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = ip->ip_src.s_addr;
	sin.sin_port = htons(113);

	if (connect(sock, (struct sockaddr *) &sin, sizeof(sin)) != 0)
		goto ident_fail;

	snprintf(buf, sizeof(buf), "%d , %d\r\n",
		ntohs(tcp->th_sport), ntohs(tcp->th_dport));

	blen = strlen(buf);
	if (send(sock, buf, blen, 0) != blen)
		goto ident_fail;

	len = recv(sock, buf, sizeof(buf), 0);
	if (len < 1)
		goto ident_fail;

	buf[len] = '\0';

	if (sscanf(buf, "%*u , %*u : USERID :%*[^:]:%64s", temp) == 1) {
		mysyslog(PRIORITY, "TCP: %s connection attempt from %s@%s:%u",
			serv_lookup(tcp->th_dport, "tcp", sbuf, sizeof(sbuf)), temp,
			host_lookup(&ip->ip_src, tcp_res(), lbuf, sizeof(lbuf)),
			ntohs(tcp->th_sport));

		free(data);
		close(sock);
		return (NULL);
	}

ident_fail:
	mysyslog(PRIORITY, "TCP: %s connection attempt from %s:%u",
		serv_lookup(tcp->th_dport, "tcp", sbuf, sizeof(sbuf)),
		host_lookup(&ip->ip_src, tcp_res(), lbuf, sizeof(lbuf)),
		ntohs(tcp->th_sport));

	free(data);
	close(sock);
	return (NULL);
}

bool is_listening(port_t port) {
	FILE *fp;
	port_t lport;
	u_int mode;
	u_char buf[1024];
	bool found = false;
	
	port = htons(port);

	fp = fopen(TCP_DATA, "r");
	if (fp == NULL)
		return (false);

	while (get_line(fp, buf, sizeof(buf) - 1) != EOF) {
		if (sscanf(buf, TCP_FORMAT, (u_int *) &lport, &mode) != 2)
			continue;
		if (lport == port) {
			found = true;
			break;
		}
	}

	return (found);
}
#endif /* __linux__ */

/* vim:ts=4:sw=8:tw=0 */
